#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import argparse
import os
import socket
import time
from contextlib import suppress

parser = argparse.ArgumentParser()
parser.add_argument('operation', choices=['upgrade', 'rollback'], nargs='?')
parser.add_argument('status', choices=['started', 'finished', 'failed'], nargs='?')
parser.add_argument('error_message', nargs='?')
args = parser.parse_args()

collectd_path = '/tmp/collectd_sock'
PLUGIN_NAME = 'upgrade_rollback'
tm = time.time()

if args.operation is not None \
   and args.status is not None \
   and os.path.exists(collectd_path):
    message = '{} {}'.format(args.operation, args.status)
    severity = 'WARNING'
    if args.status == 'failed' and args.error_message is not None:
        message = '{} {} with error: {}'.format(args.operation, args.status, args.error_message)
        severity = 'FAILURE'
    try:
        with open('/config/collectd-hostname', 'r') as f:
            host = f.readline().strip()
    except:
        host=socket.gethostname()

    msg = "PUTNOTIF type=\"\" plugin={} host={} severity={} time={} message=\"{}\"\n".format(
        PLUGIN_NAME, host, severity, str(tm), message)

    with suppress(Exception):
        cl = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        cl.connect(collectd_path)
        cl.sendall(msg.encode('utf-8'))
        cl.setblocking(0)
        ch = cl.recv(1)
        while ch != "\n":
            ch = cl.recv(1)

        cl.close()
