#!/usr/bin/python3

# Sample python plugin
# For more information see https://collectd.org/documentation/manpages/collectd-python.5.shtml

import sys
import time
from math import pi, sin
import os
import socket
import threading
import collectd
import logging
import json
import subprocess
import requests
from systemd import journal
import time
import sysrepo as sr

collectd_path = '/tmp/collectd_sock'
PLUGIN_NAME = 'k8s_notification'
#tm = time.time()

class pod_status_entry(object):
    pod_name = ""
    pod_status = ""
    timestamp = time.time()
    def __init__(self, name, status, timestamp):
        self.pod_name = name
        self.pod_status = status
        self.timestamp = timestamp
        
class PodStatus(object):
    def __init__(self):
        collectd.info("plugin load: k8s_notification successfully loaded.")
        self.PodStatusDB = list()
        self.VmStatusDB = list()
        
    def print_pod_stats_db(self, time):
        log("------  pod status list, number of entries: " + str(len(self.PodStatusDB)) + "  " + str(time) + "  -------")
        for entry in range(len(self.PodStatusDB)):
            collectd.info("%s: %s %s %s" % (PLUGIN_NAME, self.PodStatusDB[entry].pod_name, self.PodStatusDB[entry].pod_status, self.PodStatusDB[entry].timestamp))

        log("------  vm status list, number of entries: " + str(len(self.VmStatusDB)) + "  " + str(time) + "  -------")
        for entry in range(len(self.VmStatusDB)):
            collectd.info("%s: %s %s %s" % (PLUGIN_NAME, self.VmStatusDB[entry].pod_name, self.VmStatusDB[entry].pod_status, self.PodStatusDB[entry].timestamp))

    def send_notification(self, pod_name, state, namespace=None):
        if namespace:
            message = 'Status of %s changed to %s in %s' % (pod_name, state, namespace)
        else:
            message = 'Status of %s changed to %s' % (pod_name, state)
        severity = 1
        if state == 'Running':
            severity = 4
        try:
            with open('/config/collectd-hostname', 'r') as f:
                host = f.readline().strip()
        except:
            host=socket.gethostname()
        notif = collectd.Notification(plugin=PLUGIN_NAME,
                                      plugin_instance=pod_name,
                                      type="gauge",
                                      severity=severity,
                                      message=message,
                                      host=host)
        notif.dispatch()
    
    def read_function(self):
        collectd.info("k8s_notification plugin : read called")
        current_time = time.time()
        
        self.print_pod_stats_db(current_time)
        all_pods = ""
        all_vms = ""
        
        try:
            cmd = "kubectl get pods -A  | grep -v \"monitoring \""
            all_pods = (subprocess.check_output("export KUBECONFIG=/etc/kubernetes/admin.conf; %s" %(cmd), shell=True)).decode('utf8', errors='ignore')
            
            pods = set()
            if all_pods != "":
                for line in all_pods.split('\n')[1:]:
                    if line != "":
                        pod_name = line.split()[1]
                        #log(pod_name)
                        pod_status = line.split()[3]
                        pod_namespace = line.split()[0]
                        #log(pod_status)
                        pods.add(pod_name)
                        
                        
                        found = False
                        for i in range(len(self.PodStatusDB)):
                            if self.PodStatusDB[i].pod_name == pod_name:
                                found = True
                                break
        
                        if not found:
                            collectd.info("%s: Adding entry for pod: %s state: %s" % (PLUGIN_NAME, pod_name, pod_status))
                            self.PodStatusDB.append(pod_status_entry(pod_name, pod_status, current_time))
                            self.send_notification(pod_name, pod_status, pod_namespace)
                        else:
                            if self.PodStatusDB[i].pod_status != pod_status:
                                collectd.info("%s: Sending notification for pod: %s %s -> %s" % (PLUGIN_NAME, pod_name, self.PodStatusDB[i].pod_status, pod_status))
                                self.send_notification(pod_name, pod_status, pod_namespace)
                                self.PodStatusDB[i].pod_status = pod_status

        except Exception as e:
            collectd.error("k8s_notification plugin: read function failed - {}".format(e))
            
        try:        
            cmd = "kubectl get vm | grep -v No\ resources\ found  | grep -v NAME"
            all_vms = (subprocess.check_output("export KUBECONFIG=/etc/kubernetes/admin.conf; %s" %(cmd), shell=True)).decode('utf8', errors='ignore')

            vms = set()
            if all_vms != "":
                for line in all_vms.split('\n'):
                    if line != "":
                        vm_name = line.split()[0]
                        #log(vm_name)
                        vm_status = line.split()[2]
                        #log(vm_status)
                        vms.add(vm_name)

                        found = False
                        for i in range(len(self.VmStatusDB)):
                            if self.VmStatusDB[i].pod_name == vm_name:
                                found = True
                                break
        
                        if not found:
                            collectd.info("%s: Adding entry for vm: %s state: %s" % (PLUGIN_NAME, vm_name, vm_status))
                            self.VmStatusDB.append(pod_status_entry(vm_name, vm_status, current_time))
                            self.send_notification(vm_name, vm_status)
                        else:
                            if self.VmStatusDB[i].pod_status != vm_status:
                                collectd.info("%s: Sending notification for vm: %s %s -> %s" % (PLUGIN_NAME, vm_name, self.VmStatusDB[i].pod_status, vm_status))
                                self.send_notification(vm_name, vm_status)
                                self.VmStatusDB[i].pod_status = vm_status

            # clear database - delete stale entries
            for entry in self.PodStatusDB:
                if entry.pod_name not in pods:
                    collectd.info("%s: %s %s" % (PLUGIN_NAME, "Removing entry:", entry.pod_name))
                    self.PodStatusDB.remove(entry)
    
            for entry in self.VmStatusDB:
                if entry.pod_name not in vms:
                    collectd.info("%s: %s %s" % (PLUGIN_NAME, "Removing entry:", entry.pod_name))
                    self.VmStatusDB.remove(entry)

        except Exception as e:
            collectd.error("k8s_notification plugin: read function failed: {}".format(e))

                    
def log(param):
    """
    Log messages to either collectd or stdout depending on how it was called.

    :param param: the message
    :return:  None
    """

    if __name__ != '__main__':
        collectd.info("%s: %s" % (PLUGIN_NAME, param))
    else:
        sys.stderr.write("%s\n" % param)
   
if __name__ != "__main__":
    # when running inside plugin register each callback
    inst = PodStatus()
    #collectd.register_init(inst.init)
    #collectd.register_config(inst.configure_callback)
    collectd.register_read(inst.read_function)
