#!/bin/bash
exec 1> >(systemd-cat -p debug -t eth_up_down_vpp_process) 2>&1
set -o xtrace
set -o nounset
set -o pipefail
set -o errexit
interface=${1}
physical=0
self=${BASH_SOURCE[0]}
is_vlan=0
if_states_dir="/run/ifstate"

if grep -q data <<<"$interface"; then
  is_vlan=$(sysrepocfg -X -x "/ucpe:config/interfaces:interface[name='${interface}']/type" -f json | grep vlan) || true
  if [ -z "$is_vlan" ]; then
    physical=1
    linkstat=$(sysrepocfg -X -x "/ucpe:config/interfaces:interface[name='${interface}']/link-state-propagation" -f json | grep link-state-propagation) || true
    if [ -n "$linkstat" ]; then
      if grep -q false <<<"$linkstat"; then
        tracestat=$(sysrepocfg -X -x "/ucpe:config/interfaces:interface[name='${interface}']/tracehost/trace-state-propagation" -f json | grep trace-state-propagation) || true
        if grep -q false <<<"$tracestat"; then
          echo "${interface} linkstat disabled"
          exit 0
        fi
      fi
    fi
  fi
elif grep -q lte <<<"$interface"; then
  physical=1
elif grep -q wifi <<<"$interface"; then
  physical=1
elif grep -q ppp <<<"$interface"; then
  physical=1
fi

for pid in $(pgrep -f "${self} ${interface}" | grep -v ${interface}.); do
    if [ $pid != $$ ]; then
        process=$(ps -p ${pid} -o comm= || true)
        [[ "${process}" == "eth_up_down_vpp" ]] && kill $pid
    fi
done

state=${2}

# Save link state for every interface in /run/ifstate
# or use states from /run/network/ifstate.<interface>
if [ ! -d ${if_states_dir} ]; then
  sudo mkdir ${if_states_dir}
fi

if [[ ${physical} -eq 1 ]] ; then
  linkstat=$(sysrepocfg -X -x "/ucpe:config/interfaces:interface[name='${interface}']/link-state-propagation" -f json | grep link-state-propagation) || true
  if [ -n "$linkstat" ]; then
    if grep -q true <<<"$linkstat"; then
      if [ "${state,,}" == "up" ]; then
        sudo touch ${if_states_dir}/br-${interface}.up
        sudo rm -f ${if_states_dir}/br-${interface}.down
        sudo touch /config/_reboot/ifup/br-${interface}
      else
        sudo touch ${if_states_dir}/br-${interface}.down
        sudo rm -f ${if_states_dir}/br-${interface}.up
        sudo touch /config/_reboot/ifdown/br-${interface}
      fi
    fi
  fi
fi
#
exit 0
